searchData={"items":[{"type":"module","title":"EctoSync","doc":"A Cache updater and router for events emitted when database entries are updated. Subscribers can provide a list of records that they want to receive updates on. Additionally, they can provide a function that will act as a means of authorization on the updates they should get.\n\nUsing the subscribe function a process can subscribe to all messages for a given struct.","ref":"EctoSync.html"},{"type":"function","title":"EctoSync.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"EctoSync.html#child_spec/1"},{"type":"type","title":"EctoSync.schema_or_list_of_schemas/0","doc":"","ref":"EctoSync.html#t:schema_or_list_of_schemas/0"},{"type":"function","title":"EctoSync.start_link/1","doc":"","ref":"EctoSync.html#start_link/1"},{"type":"function","title":"EctoSync.subscribe/1","doc":"Subscribe to Ecto.Schema(s) provided. The input can be one of following:\n - an Ecto.Schema struct, \n - a list of Ecto.Schema struct, \n - an EctoWatch identifier.\n\nWhen an Ecto.Schema struct or list of structs is provided, the process is subscribed to all `:updated` and `:deleted` events for the Ecto.Schema that represents the struct.","ref":"EctoSync.html#subscribe/1"},{"type":"function","title":"Examples - EctoSync.subscribe/1","doc":"iex> defmodule Test do\n    ...>   use Ecto.Schema\n    ...>   schema do\n    ...>     field :name, :string\n    ...>   end\n    ...> end\n\n    iex> EctoSync.subscribe(Test)\n    [{{Test, :inserted}, nil}, {{Test, :updated}, nil}, {{Test, :deleted}, nil}]\n\n    iex> EctoSync.subscribe(%Test{id: 1})\n    [{{Test, :updated}, 1}, {{Test, :deleted}, 1}]","ref":"EctoSync.html#subscribe/1-examples"},{"type":"function","title":"EctoSync.subscribe/2","doc":"","ref":"EctoSync.html#subscribe/2"},{"type":"type","title":"EctoSync.subscriptions/0","doc":"","ref":"EctoSync.html#t:subscriptions/0"},{"type":"function","title":"EctoSync.subscriptions/2","doc":"Returns a list of pids that are subscribed to the given watcher identifier.","ref":"EctoSync.html#subscriptions/2"},{"type":"function","title":"EctoSync.sync/2","doc":"Performs the actual syncing of a given value. Based on the input and the event, certain behaviour can be expected.\n\n## `:inserted` event\n|struct|input|output|\n|------------|-----|------|\n|`%Post{id: 1}`|`[]`|` [%Post{id: 1}]`|\n|`%Post{id: 2}`|`[%Post{id: 1}]`|` [%Post{id: 1}, %Post{id: 2}]`|\n|`%Comment{id: 1, post_id: 1}`|`[%Post{id: 1, comments: []}]`|`[%Post{id: 1, comments: [%Comment{id: 1}]}]`|\n\n## `:updated` event\n\n|struct|input|output|\n|------------|-----|------|\n|`%Post{id: 1, name: \"Updated name\"}`|`%Post{id:1, name: \"Name\"}`|`%Post{id: 1}, name: \"Updated name\"}`|\n|`%Post{id: 2}`|`[%Post{id: 1}]`|`[%Post{id: 1}, %Post{id: 2}]`|\n|`%Comment{id: 1, post_id: 1}`|`[%Post{id: 1, comments: []}]`|` [%Post{id: 1, comments: [%Comment{id: 1}]}]`|\n|`%Comment{id: 1, post_id: 2}`|`[%Post{id: 1, comments: [%Comment{id: 1}]}, %Post{id: 2, comments: []}]`|` [%Post{id: 1, comments: []}, %Post{id: 2, comments: [%Comment{id: 1}]}]`|\n\n## `:deleted` event\n\n|struct|input|output|\n|------------|-----|------|\n|`%Post{id: 1}`|`[]`|`[]`|\n|`%Post{id: 1}`|`%Post{id: 1}`|`%Post{id: 1}`|\n|`%Post{id: 2}`|`[%Post{id: 1}, %Post{id: 2}]`|`[%Post{id: 1}]`|\n|`%Comment{id: 1, post_id: 1}`|` [%Post{id: 1, comments: [%Comment{id: 1}]}]`|`[%Post{id: 1, comments: []}]`|","ref":"EctoSync.html#sync/2"},{"type":"function","title":"EctoSync.unsubscribe/2","doc":"","ref":"EctoSync.html#unsubscribe/2"},{"type":"function","title":"EctoSync.watchers/2","doc":"Create watcher specifications for a given schema.\n\n  * `schema` - The Ecto.Schema to subscribe to.","ref":"EctoSync.html#watchers/2"},{"type":"function","title":"Options - EctoSync.watchers/2","doc":"- `:extra_columns`, which extra columns should be included  \n  - `:assocs`, a preload like keyword list of associations to subscribe to. \n    If assocs are specified in the options, the necessary extra_columns will be added \n    or merged to the `:extra_columns` option.","ref":"EctoSync.html#watchers/2-options"},{"type":"function","title":"Examples - EctoSync.watchers/2","doc":"Assuming the same schemas are present as in the Use Cases page.\n\n```elixir\n# Generate events for a schema without associations\nwatchers(MyApp.User)\n# => [\n#      {MyApp.User, :inserted, []},\n#      {MyApp.User, :updated, []},\n#      {MyApp.User, :deleted, []}\n#    ]\n\n# Generate events for all associations\nwatchers(User, assocs: [posts: [:comments, :tags, :labels]])\n# => Includes events for:\n# [\n#   {Label, :deleted, [extra_columns: []]},\n#   {Label, :inserted, [extra_columns: []]},\n#   {Label, :updated, [extra_columns: []]},\n#   {Person, :deleted, [extra_columns: []]},\n#   {Person, :inserted, [extra_columns: []]},\n#   {Person, :updated, [extra_columns: []]},\n#   {Post, :deleted, [extra_columns: [:person_id]]},\n#   {Post, :deleted, [extra_columns: [:post_id]]},\n#   {Post, :inserted, [extra_columns: [:person_id]]},\n#   {Post, :inserted, [extra_columns: [:post_id]]},\n#   {Post, :updated, [extra_columns: [:person_id]]},\n#   {Post, :updated, [extra_columns: [:post_id]]},\n#   {PostsTags, :deleted, [extra_columns: [:tag_id, :post_id]]},\n#   {PostsTags, :inserted, [extra_columns: [:tag_id, :post_id]]},\n#   {PostsTags, :updated, [extra_columns: [:tag_id, :post_id]]},\n#   {Tag, :deleted, [extra_columns: []]},\n#   {Tag, :inserted, [extra_columns: []]},\n#   {Tag, :updated, [extra_columns: []]}\n# ]\n```","ref":"EctoSync.html#watchers/2-examples"},{"type":"function","title":"EctoSync.watchers/3","doc":"See `watchers/2`.","ref":"EctoSync.html#watchers/3"},{"type":"extras","title":"EctoSync","doc":"# EctoSync\n\nSubscribe to events emitted by EctoWatch, and cache the changed row. Use the `sync` function\nto update an in-memory row or list of rows using the cached version.","ref":"readme.html"},{"type":"extras","title":"Use cases - EctoSync","doc":"","ref":"readme.html#use-cases"},{"type":"extras","title":"Schema: - EctoSync","doc":"Post <-> PostsTags <-> Tag","ref":"readme.html#schema"},{"type":"extras","title":"Subscribe to alterations of assocs of loaded rows (in a LiveView for example): - EctoSync","doc":"#### application.ex\n```\nchildren = [\n  ...,\n  {EctoSync,\n   repo: MyRepo,\n   pub_sub: MyPubSub,\n   watchers: EctoSync.watchers(Posts, assocs: [:tags])\n  },\n  ...\n]\n```\n#### PostLive.ex\n```\nalias MyApp.Posts.Post\nalias MyApp.Posts.Tag\n\ndef mount(params, session, socket) do\n  posts = list_posts(preload: [:tags])\n  if connected?(socket) do\n    EctoSync.subscribe(Post, assocs: [:tags])\n  end\n\n  {:ok, assign(socket, posts: posts)}\nend\n\ndef handle_info({{PostsTags, _event}, _} = sync_args, socket) do\n  {:ok, posts} = EctoSync.sync(socket.assigns.posts, sync_args)\n  {:noreply, assign(socket, posts: posts)}\nend\n```\n1. When a Tag row is updated, all posts that reference that tag will be updated.\n2. When a PostsTags row is inserted, the tag will automatically be added to the corresponding post.\n\nThe Tag is loaded from the database once for each event handled by EctoSync. \nThe result is then cached and this cached version will be used when other processes call\n`EctoSync.sync/2` with the args provided in `handle_info`.","ref":"readme.html#subscribe-to-alterations-of-assocs-of-loaded-rows-in-a-liveview-for-example"},{"type":"extras","title":"Subscribe to events of rows that are used as assocs: - EctoSync","doc":"#### PostForm.ex\n```\ndef mount(params, session, socket) do\n  tags = list_tags()\n  if connected?(socket) do\n    EctoSync.subscribe(tags)\n  end\n\n  {:ok, assign(socket, tags: tags)}\nend\n\ndef render(assigns) do\n  ~F\"\"\"\n  <.simple_form for={@form}>\n     \n     \n       {tag.name} \n     \n  </.simple_form>\n  \"\"\"\nend\n\ndef handle_info({{Tag, _event}, _} = sync_args, socket) do\n  {:ok, tags} = EctoSync.sync(socket.assigns.tags, sync_args)\n  {:noreply, assign(socket, tags: tags)\nend\n```","ref":"readme.html#subscribe-to-events-of-rows-that-are-used-as-assocs"},{"type":"extras","title":"Installation - EctoSync","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `ecto_sync` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_sync, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Modify lib/my_app/application.ex - EctoSync","doc":"```\nchildren = [\n  ...,\n  {EctoSync,\n   repo: MyRepo,\n   pub_sub: MyPubSub,\n   watchers:\n     EctoSync.watchers(Schema)\n     |> EctoSync.watchers(OtherSchema, assocs: [:some_assocs])\n     |> EctoSync.watchers(AnotherSchema, extra_columns: [:test_id])},\n  ]\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#modify-lib-my_app-application-ex"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}